# -*- coding: utf-8 -*-
#    Copyright 2005 Spike^ekipS <spikeekips@gmail.com>
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from twisted.internet import defer
from twisted.python import failure
from twisted.conch.ssh import filetransfer
import gobject, exceptions

import FileBrowser, Progress

class SSH :

	QUEUE = list()
	DATA = dict()
	TREESTORE = None
	FILE_TRANSFER = None

	def __init__ (self, filetransfer) :
		self.FILE_TRANSFER = filetransfer

	def append (self, uid, **kwargs) :
		if not kwargs.has_key("filepath-remote") or \
				not kwargs.has_key("filepath-local") or \
				not kwargs.has_key("method") or \
				not kwargs.has_key("iter") :
			raise RuntimeError, \
				"filepath-remote, filepath-local, iter must be given."

		self.DATA[uid] = kwargs
		self.QUEUE.append(uid)

	def next (self) :
		if len(self.QUEUE) < 1 :
			# if there is no item in queue, restart queue.
			self.stopRunning()
			self.startRunning()
			return

		self.startNext(self.QUEUE[0])

	def __start (self, uid) :
		__progress = Progress.SSH(self.DATA[uid]["method"], \
			self.TREESTORE, self.DATA[uid]["iter"], self.FILE_TRANSFER)

		__filepath_local = \
			self.FILE_TRANSFER.getRealPathLocal( \
				self.DATA[uid]["filepath-local"], "is-file" \
			)

		__filepath_remote = \
			self.FILE_TRANSFER.getRealPathRemote( \
				self.DATA[uid]["filepath-remote"], "is-file" \
			)

		if self.DATA[uid]["method"] == "get" :
			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_GET( \
				__filepath_remote, __filepath_local, __progress)
		elif self.DATA[uid]["method"] == "put" :
			d = self.FILE_TRANSFER.FILE_TRANSFER.cmd_PUT( \
				__filepath_local,__filepath_remote, __progress)

		return d

	def start (self, uid) :
		d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid)
		else :
			d.addCallback(self.on_finished, uid)
			d.addErrback(self.on_error, uid)

	def startNext (self, uid) :
		d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid, "keep going")
		else :
			d.addCallback(self.on_finished, uid, "keep going")
			d.addErrback(self.on_error, uid, "keep going")

		return False

	HANDLER_RUNNING = None
	def startRunning (self) :
		if self.HANDLER_RUNNING :
			return

		# after 1 second.
		self.HANDLER_RUNNING = gobject.timeout_add(1000, self.__startRunning)

	def __startRunning (self) :
		try :
			uid = self.QUEUE[0]
		except IndexError :	
			return True
		else :
			d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid, "keep going")
		else :
			d.addCallback(self.on_finished, uid, "keep going")
			d.addErrback(self.on_error, uid, "keep going")

		return False

	def stopRunning (self) :
		gobject.source_remove(self.HANDLER_RUNNING)
		self.HANDLER_RUNNING = None

	def on_finished (self, result, uid, keepGoing=False) :
		del(self.QUEUE[0])

		if keepGoing : self.next()

	def on_error (self, f, uid, keepGoing=False) :
		try :
			del(self.QUEUE[0])
		except :
			pass

		self.FILE_TRANSFER.errBackRemote(f, "transfer")
		if keepGoing : self.next()

	def stop (self, uid) :
		try :
			__idx = self.QUEUE.index(uid)
		except ValueError :
			pass
		else :
			del(self.QUEUE[__idx])
			if __idx == 0 : self.FILE_TRANSFER.FILE_TRANSFER.stopTransfer()

class Local :

	QUEUE = list()
	DATA = dict()
	TREESTORE = None
	FILE_TRANSFER = None

	def __init__ (self, filetransfer) :
		self.FILE_TRANSFER = filetransfer

	def append (self, uid, **kwargs) :
		if not kwargs.has_key("filepath-src") or \
				not kwargs.has_key("filepath-dst") or \
				not kwargs.has_key("method") or \
				not kwargs.has_key("iter") :
			raise RuntimeError, \
				"filepath-src, filepath-dst, iter must be given."

		self.DATA[uid] = kwargs
		self.QUEUE.append(uid)

	def next (self) :
		if len(self.QUEUE) < 1 :
			# if there is no item in queue, restart queue.
			self.stopRunning()
			self.startRunning()
			return

		self.startNext(self.QUEUE[0])

	def __start (self, uid) :
		#__progress = Progress.Local(self.DATA[uid]["method"], \
		#	self.TREESTORE, self.DATA[uid]["iter"], self.FILE_TRANSFER)

		if self.DATA[uid]["method"] == "get" :
			__dst = self.DATA[uid]["filepath-dst"]
			__src = self.DATA[uid]["filepath-src"]
		elif self.DATA[uid]["method"] == "put" :
			__src = self.DATA[uid]["filepath-dst"]
			__dst = self.DATA[uid]["filepath-src"]

		d = defer.maybeDeferred(self.__copy, __src, __dst)

		return d

	def __copy (self, src, dst) :
		if __OS__.is_dir(src) :
			try :
				shutil.copytree( \
					src,
					os.path.join( \
						os.path.join(dst), \
						os.path.basename(src) \
					) \
				)
			except :
				self.FILE_TRANSFER.errBackLocal(__OS__.ERROR_IOERROR)
				return False
		else :
			try :
				shutil.copy(src, dst)
			except :
				self.FILE_TRANSFER.errBackLocal(__OS__.ERROR_IOERROR)
				return False

	def start (self, uid) :
		d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid)
		else :
			d.addCallback(self.on_finished, uid)
			d.addErrback(self.on_error, uid)

	def startNext (self, uid) :
		d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid, "keep going")
		else :
			d.addCallback(self.on_finished, uid, "keep going")
			d.addErrback(self.on_error, uid, "keep going")

		return False

	HANDLER_RUNNING = None
	def startRunning (self) :
		if self.HANDLER_RUNNING :
			return

		self.HANDLER_RUNNING = gobject.timeout_add(2000, self.__startRunning)

	def __startRunning (self) :
		try :
			uid = self.QUEUE[0]
		except IndexError :	
			return True
		else :
			d = self.__start(uid)

		if isinstance(d, failure.Failure) :
			return self.on_error(d, uid, "keep going")
		else :
			d.addCallback(self.on_finished, uid, "keep going")
			d.addErrback(self.on_error, uid, "keep going")

		return False

	def stopRunning (self) :
		gobject.source_remove(self.HANDLER_RUNNING)
		self.HANDLER_RUNNING = None

	def on_finished (self, result, uid, keepGoing=False) :
		del(self.QUEUE[0])

		if keepGoing : self.next()

	def on_error (self, f, uid, keepGoing=False) :
		try :
			del(self.QUEUE[0])
		except :
			pass

		self.FILE_TRANSFER.errBackRemote(f, "transfer")
		if keepGoing : self.next()

	def stop (self, uid) :
		try :
			__idx = self.QUEUE.index(uid)
		except ValueError :
			pass
		else :
			del(self.QUEUE[__idx])
			#if __idx == 0 : self.FILE_TRANSFER.FILE_TRANSFER.stopTransfer()

"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "FileTransferQueue.py"


