# -*- coding: utf-8 -*-
#    Copyright (C) 2005 Spike^ekipS <spikeekips@gmail.com>
#
#    http://vee.kldp.net
#
#       This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import gtk
import gtk.glade
import Conf, i18n

_ = i18n.gettext()

class __WIDGET__ :

	xml = None
	HANDLER_METHOD = dict()
	HANDLER_SIGNAL = dict()

	def __init__ (self, WINDOWNAME, \
				ROOTWIDGET=None, handler=dict(), handler_signal=dict(), \
				glade_file=None, messages=dict() \
			) :

		self.xml = gtk.glade.XML(glade_file, WINDOWNAME)

		self.WIDGET_SELF = self.getWidget(WINDOWNAME)
		self.WIDGET_ROOT = ROOTWIDGET

		translate(self.xml, messages)
		self.initialize()

		# assign handler.
		self.connect_handler(handler)
		self.connect_signal(handler_signal)

	def initialize (self) :
		pass

	def connect_handler (self, handler=dict()) :
		self.HANDLER_METHOD["on_quit"] = self.on_quit
		self.HANDLER_METHOD["on_close"] = self.on_close

		if type(handler) is dict and len(handler) > 0 :
			[self.HANDLER_METHOD.update({i : eval("self.%s" % j)}) for i, j in handler.items()]

		self.xml.signal_autoconnect(self.HANDLER_METHOD)

	def connect_signal (self, handler=dict()) :
		self.HANDLER_SIGNAL["size_request"] = self.on_size_request

		if type(handler) is dict and len(handler) > 0 :
			[self.HANDLER_SIGNAL.update({i : eval("self.%s" % j)}) for i, j in handler.items()]

		for i, j in self.HANDLER_SIGNAL.items() :
			try :
				self.WIDGET_SELF.connect(i, j)
			except AttributeError :
				continue

	# INTERNAL METHOD ##################################################
	def getWidget (self, widget_name, xml=None) :
		if not xml : xml = self.xml

		try :
			return xml.get_widget(widget_name)
		except AttributeError :
			return None

	# DEFAULT HANDLER ##################################################
	def on_size_request (self, widget, requisition) :
		try :
			self.WIDGET_SELF.set_data("window-size", self.WIDGET_SELF.get_size())
		except AttributeError :
			pass

	def on_close (self, widget) :
		self.WIDGET_SELF.destroy()
		return

	def on_quit (self, widget) :
		gtk.main_quit()

class WIDGET (__WIDGET__) :

	def __init__ (self, widget_name) :

		if not Conf.MAP_WIDGET.has_key(widget_name) :
			raise RuntimeError, \
				"widget_name, %s does not registered in Conf.MAP_WIDGET" % \
				widget_name

		# check glade file is exits,

		if not Conf.FILES_GLADE.has_key( \
				Conf.MAP_WIDGET[widget_name]["glade-file"]) :
			raise RuntimeError, \
				"glade_file, %s.glade does not registered." % widget_name

		__glade_file = \
			Conf.FILES_GLADE[Conf.MAP_WIDGET[widget_name]["glade-file"]]

		__WIDGET__.__init__( \
			self, \
			widget_name, \
			Conf.MAP_WIDGET[widget_name]["root"], \
			Conf.MAP_WIDGET[widget_name]["handler"], \
			Conf.MAP_WIDGET[widget_name]["handler-signal"], \
			__glade_file, \
			Conf.MAP_WIDGET[widget_name]["message"], \
		)

"""
Functions
"""
def translate (xml, messsage) :
	if messsage is None : return

	__tooltip = gtk.Tooltips()
	for __name, __tuple in messsage.items() :
		__widget = xml.get_widget(__name)
		if __widget is None :
			continue

		for j in __tuple :
			if j[3] == "label" :
				if j[0] in ["GtkImageMenuItem", "GtkCheckMenuItem", ] :
					__func = __widget.get_children()[0].set_label

				else :
					try :
						__func = xml.get_widget(__name).set_label
					except :
						continue

				try :
					__func(_(j[2]))
				except AttributeError :
					continue

			elif j[3] == "title" :
				if j[0] == "GtkDialog" : # set also tooltip.
					__widget.set_title(_(j[2]))
					continue

			elif j[3] == "tooltip" :

				if j[0] == "GtkToggleToolButton" :
					"""
					To attach tooltip in toolbutton
						------------------------------
							toolbar = gtk.Toolbar()
							toolbar.set_tooltips(True)
						------------------------------

						old style is,
						------------------------------
							tooltip = gtk.ToolTips()
							tooltip.set_tip(button_widget, "hi")
						------------------------------

						new style is,
						------------------------------
							tooltip = gtk.ToolTips()
							button_widget.set_tooltip(tooltip, "hi")
						------------------------------
					"""
					__tooltip.set_tip(__widget, _(j[2]))
				else :
					__tooltip.set_tip(__widget, _(j[2]))

def get_widget_from_xml0 (glade_file, xmlname, widget_name="") :
	xml = gtk.glade.XML(glade_file, xmlname)
	widget_self = xml.get_widget(xmlname)

	if not widget_name :
		return widget_self
	else :
		return gtk.glade.XML(Conf.FILE_GLADE, xmlname, widget_name)

def get_widget (widget_name) :
	if not Conf.MAP_WIDGET.has_key(widget_name) :
		raise RuntimeError, \
			"widget_name, %s does not registered in Conf.MAP_WIDGET" % \
			widget_name

	__glade_file = Conf.FILES_GLADE[Conf.MAP_WIDGET[widget_name]["glade-file"]]

	return get_widget_from_xml( \
		widget_name, \
		__glade_file, \
	)

def get_widget_from_xml (WINDOWNAME, glade_file=None) :

	if not glade_file : glade_file = Conf.FILE_GLADE

	xml = gtk.glade.XML(glade_file, WINDOWNAME)

	return (xml, xml.get_widget(WINDOWNAME), )

def set_title (widget, gconf_client, strings, isFilepath=True) :
	__title_format = gconf_client.getValue("window_title")

	if not isFilepath :
		__title = "%s : %s" % (strings, _("VEE"))
	else :
		__path_short = os.path.basename(strings)
		__path_full = strings

		__title = \
			__title_format.replace( \
				"%F", __path_full).replace( \
				"%t", __path_short).replace( \
				"%V", _("VEE"))

	widget.set_title(__title)

def get_window_geometry (window) :
	"""
	@return : (width, height, x position, y position,)
	"""
	#(w, h) = window.get_size()
	(w, h) = window.get_data("window-size")
	(x, y) = window.get_position()

	return (w, h, x, y)

def set_window_geometry (window, geometry=(100, 100, 0, 0,)) :
	"""
	Notice :
		resize() and set_size_request() will change the window
		size, but set_size_request keep the window from being
		smaller, so here i use resize().
	"""
	(w, h, x, y) = geometry

	#window.set_resize_mode(gtk.RESIZE_PARENT)
	#window.resize(w, h)

	#__dummy_window = gtk.Window()
	#__dummy_window.realize()
	#__dummy_window.show()

	if x and y : window.move(x, y)
	#window.set_size_request(w, h)
	window.resize(w, h)

	#__dummy_window.destroy()

	return

def parse_geometry (geom) :
	if geom.count("x") == 0 :
		w, h = None, None
	else :
		__wh = sre.compile("[1-9][0-9]*x[1-9][0-9]*").findall(geom)

		try :
			w, h = __wh[0].split("x")
		except IndexError :
			return None
		else :
			w, h = (int(w), int(h),)

	if geom.count("+") == 0 :
		x, y = None, None
	else :
		__xy = sre.compile("\+[1-9][0-9]*\+[1-9][0-9]*").findall(geom)

		try :
			x, y = __xy[0].replace("+", " ").split()
		except IndexError :
			return None
		else :
			x, y = (int(x), int(y),)

	return (w, h, x, y)

def remember_geometry (window, gconf_client) :
	(w, h, x, y, ) = get_window_geometry(window)

	gconf_client.setValue( \
		"latest_window_geometry", \
		"%d,%d,%d,%d" % (w, h, x, y) \
	)

	# save the state of fullscreened or not.
	#gconf_client.set_value_by_conf( \
	#	"is-fullscreen", is_fullscreened)


"""
Description
-----------


ChangeLog
---------


Usage
-----


"""

__author__ =  "Spike^ekipS <spikeekips@gmail.com>"
__version__=  "0.1"
__nonsense__ = ""

__file__ = "WIDGET.py"

